@@ -5,7 +5,7 @@ from pulp import lpSum, LpVariable, LpProblem, LpMinimize
 from pm4py.algo.conformance.tree_alignments.variants.approximated.calculate_a_sa_ea_sets import \
     initialize_a_sa_ea_tau_sets
 from pm4py.algo.conformance.tree_alignments.variants.approximated.utilities import calculate_optimal_alignment, \
-    concatenate_traces, trace_to_list_of_str, add_fitness_and_cost_info_to_alignments
+    concatenate_traces, trace_to_list_of_str, add_fitness_and_cost_info_to_alignments, AlignmentNoneException, EfficientTree
 from pm4py.objects.process_tree.util import get_process_tree_height, process_tree_to_binary_process_tree
 from pm4py.objects.petri.align_utils import SKIP
 from pm4py.objects.process_tree.process_tree import ProcessTree
@@ -18,6 +18,8 @@ from pm4py.util.xes_constants import DEFAULT_NAME_KEY
 from pm4py.util import exec_utils, constants
 from pm4py.statistics.variants.log.get import get_variants_from_log_trace_idx
 from enum import Enum
+import sys
+import time
 
 
 class Parameters(Enum):
@@ -25,6 +27,8 @@ class Parameters(Enum):
     MAX_PROCESS_TREE_HEIGHT = "max_process_tree_height"
     PARAMETER_VARIANT_DELIMITER = "variant_delimiter"
     ACTIVITY_KEY = constants.PARAMETER_CONSTANT_ACTIVITY_KEY
+    PARAM_MAX_ALIGN_TIME_TRACE = "max_align_time_trace"
+    PARAM_MAX_ALIGN_TIME = "max_align_time"
 
 
 def apply_from_variants_tree_string(var_list, tree_string, parameters=None):
@@ -146,20 +150,37 @@ def __align(obj: Union[Trace, EventLog], pt: ProcessTree, max_trace_length: int
         obj = e
     assert isinstance(obj, EventLog)
     pt = process_tree_to_binary_process_tree(pt)
+    pt = EfficientTree(pt)
+    
     return __approximate_alignments_for_log(obj, pt, max_trace_length, max_process_tree_height,
                                             parameters=parameters)
 
 
 def __approximate_alignments_for_log(log: EventLog, pt: ProcessTree, max_tl: int, max_th: int,
                                      parameters=None):
+    if parameters is None:
+        parameters = {}
+
     a_sets, sa_sets, ea_sets, tau_sets = initialize_a_sa_ea_tau_sets(pt)
     variants = get_variants_from_log_trace_idx(log, parameters=parameters)
     inv_corr = {}
+
+    max_align_time = exec_utils.get_param_value(Parameters.PARAM_MAX_ALIGN_TIME, parameters,
+                                                sys.maxsize)
+    log_alignment_start_time = time.time()
+
     for i, var in enumerate(variants):
-        alignment = __approximate_alignment_for_trace(pt, a_sets, sa_sets, ea_sets, tau_sets, log[variants[var][0]],
-                                                      max_tl, max_th,
-                                                      parameters=parameters)
-        alignment = add_fitness_and_cost_info_to_alignments(alignment, pt, log[variants[var][0]])
+        this_time = time.time()
+
+        if this_time - log_alignment_start_time <= max_align_time:
+            parameters["trace_alignment_start_time"] = this_time
+            alignment = __approximate_alignment_for_trace(pt, a_sets, sa_sets, ea_sets, tau_sets, log[variants[var][0]],
+                                                          max_tl, max_th,
+                                                          parameters=parameters)
+            alignment = add_fitness_and_cost_info_to_alignments(alignment, pt, log[variants[var][0]], parameters=parameters)
+        else:
+            alignment = None
+
         for idx in variants[var]:
             inv_corr[idx] = alignment
     alignments = []
@@ -172,21 +193,40 @@ def __approximate_alignment_for_trace(pt: ProcessTree, a_sets: Dict[ProcessTree,
                                       sa_sets: Dict[ProcessTree, Set[str]], ea_sets: Dict[ProcessTree, Set[str]],
                                       tau_flags: Dict[ProcessTree, bool], trace: Trace, max_tl: int,
                                       max_th: int, parameters=None):
-    if len(trace) <= max_tl or get_process_tree_height(pt) <= max_th:
-        return calculate_optimal_alignment(pt, trace, parameters=parameters)
-    else:
-        if pt.operator == Operator.SEQUENCE:
-            return __approximate_alignment_on_sequence(pt, trace, a_sets, sa_sets, ea_sets, tau_flags, max_tl, max_th,
-                                                       parameters=parameters)
-        elif pt.operator == Operator.LOOP:
-            return __approximate_alignment_on_loop(pt, trace, a_sets, sa_sets, ea_sets, tau_flags, max_tl, max_th,
-                                                   parameters=parameters)
-        elif pt.operator == Operator.XOR:
-            return __approximate_alignment_on_choice(pt, trace, a_sets, sa_sets, ea_sets, tau_flags, max_tl, max_th,
-                                                     parameters=parameters)
-        elif pt.operator == Operator.PARALLEL:
-            return __approximate_alignment_on_parallel(pt, trace, a_sets, sa_sets, ea_sets, tau_flags, max_tl, max_th,
+    if parameters is None:
+        parameters = {}
+
+    max_align_time_trace = exec_utils.get_param_value(Parameters.PARAM_MAX_ALIGN_TIME_TRACE, parameters,
+                                                      sys.maxsize)
+
+    start_time = parameters["trace_alignment_start_time"]
+    current_time = time.time()
+
+    if current_time - start_time > max_align_time_trace:
+        # the alignment of the trace did not terminate in an useful time
+        return None
+
+    try:
+        if len(trace) <= max_tl or get_process_tree_height(pt) <= max_th:
+            return calculate_optimal_alignment(pt, trace, parameters=parameters)
+        else:
+            if pt.operator == Operator.SEQUENCE:
+                return __approximate_alignment_on_sequence(pt, trace, a_sets, sa_sets, ea_sets, tau_flags, max_tl,
+                                                           max_th,
+                                                           parameters=parameters)
+            elif pt.operator == Operator.LOOP:
+                return __approximate_alignment_on_loop(pt, trace, a_sets, sa_sets, ea_sets, tau_flags, max_tl, max_th,
                                                        parameters=parameters)
+            elif pt.operator == Operator.XOR:
+                return __approximate_alignment_on_choice(pt, trace, a_sets, sa_sets, ea_sets, tau_flags, max_tl, max_th,
+                                                         parameters=parameters)
+            elif pt.operator == Operator.PARALLEL:
+                return __approximate_alignment_on_parallel(pt, trace, a_sets, sa_sets, ea_sets, tau_flags, max_tl,
+                                                           max_th,
+                                                           parameters=parameters)
+    except AlignmentNoneException:
+        # alignment did not terminate correctly. return None
+        return None
 
 
 def __approximate_alignment_on_choice(pt: ProcessTree, trace: Trace, a_sets: Dict[ProcessTree, Set[str]],
@@ -438,9 +478,12 @@ def __approximate_alignment_on_loop(pt: ProcessTree, trace: Trace, a_sets: Dict[
 
     res = []
     for subtree, sub_trace in alignments_to_calculate:
-        res.extend(
-            __approximate_alignment_for_trace(subtree, a_sets, sa_sets, ea_sets, tau_flags, sub_trace, tl, th,
-                                              parameters=parameters))
+        align_result = __approximate_alignment_for_trace(subtree, a_sets, sa_sets, ea_sets, tau_flags, sub_trace, tl, th,
+                                              parameters=parameters)
+        if align_result is None:
+            # the alignment did not terminate correctly.
+            return None
+        res.extend(align_result)
     return res
 
 
@@ -586,9 +629,12 @@ def __approximate_alignment_on_sequence(pt: ProcessTree, trace: Trace, a_sets: D
     # calculate and compose alignments
     res = []
     for subtree, sub_trace in alignments_to_calculate:
-        res.extend(
-            __approximate_alignment_for_trace(subtree, a_sets, sa_sets, ea_sets, tau_flags, sub_trace, tl, th,
-                                              parameters=parameters))
+        align_result = __approximate_alignment_for_trace(subtree, a_sets, sa_sets, ea_sets, tau_flags, sub_trace, tl, th,
+                                              parameters=parameters)
+        if align_result is None:
+            # the alignment did not terminate correctly.
+            return None
+        res.extend(align_result)
     return res
 
 
@@ -740,9 +786,13 @@ def __approximate_alignment_on_parallel(pt: ProcessTree, trace: Trace, a_sets: D
         for trace_part in trace_parts:
             if subtree == trace_part[0]:
                 sub_trace = concatenate_traces(sub_trace, trace_part[1])
-        alignments_per_subtree[subtree] = __approximate_alignment_for_trace(subtree, a_sets, sa_sets, ea_sets,
+        align_result = __approximate_alignment_for_trace(subtree, a_sets, sa_sets, ea_sets,
                                                                             tau_flags, sub_trace, tl, th,
                                                                             parameters=parameters)
+        if align_result is None:
+            # the alignment did not terminate correctly.
+            return None
+        alignments_per_subtree[subtree] = align_result
     # compose alignments from subtree alignments
     res = []
     for trace_part in trace_parts:
