@@ -8,26 +8,13 @@ from pm4py.objects.conversion.log import constants
 from pm4py.objects.log import log as log_instance
 from pm4py.objects.log.log import EventLog, Event
 from pm4py.util import constants as pmutil
+from pm4py.util import exec_utils
 
 
 class Parameters(Enum):
-    DEEP_COPY = False
-    STREAM_POST_PROCESSING = False
-    CASE_ATTRIBUTE_PREFIX = 'case:'
-
-
-DEEPCOPY = constants.DEEPCOPY
-STREAM_POST_PROCESSING = constants.STREAM_POSTPROCESSING
-
-
-def __parse_parameters(parameters):
-    if DEEPCOPY in parameters:
-        parameters[Parameters.DEEP_COPY] = parameters[DEEPCOPY]
-    if STREAM_POST_PROCESSING in parameters:
-        parameters[Parameters.STREAM_POST_PROCESSING] = parameters[STREAM_POST_PROCESSING]
-    if pmutil.PARAMETER_KEY_CASE_ATTRIBUTE_PRFIX in parameters:
-        parameters[Parameters.CASE_ATTRIBUTE_PREFIX] = parameters[pmutil.PARAMETER_KEY_CASE_ATTRIBUTE_PRFIX]
-    return parameters
+    DEEP_COPY = constants.DEEPCOPY
+    STREAM_POST_PROCESSING = constants.STREAM_POSTPROCESSING
+    CASE_ATTRIBUTE_PREFIX = "case_attribute_prefix"
 
 
 def __postprocess_stream(list_events):
@@ -54,6 +41,10 @@ def __postprocess_stream(list_events):
             if (type(event[k]) is float or type(event[k]) is int) and math.isnan(event[k]):
                 del event[k]
                 continue
+            ev_str = str(event[k]).lower()
+            if ev_str == "none" or ev_str == "null" or len(ev_str) == 0:
+                del event[k]
+                continue
     return list_events
 
 
@@ -77,20 +68,21 @@ def apply(log, parameters=None):
       log : :class:`pm4py.log.log.EventLog`
           An Event stream
       """
-    parameters = dict() if parameters is None else __parse_parameters(parameters)
+    if parameters is None:
+        parameters = {}
+
+    stream_post_processing = exec_utils.get_param_value(Parameters.STREAM_POST_PROCESSING, parameters, False)
+    case_pref = exec_utils.get_param_value(Parameters.CASE_ATTRIBUTE_PREFIX, parameters, 'case:')
+    enable_deepcopy = exec_utils.get_param_value(Parameters.DEEP_COPY, parameters, False)
+
     if isinstance(log, pandas.core.frame.DataFrame):
         list_events = log.to_dict('records')
-        if Parameters.STREAM_POST_PROCESSING in parameters and parameters[Parameters.STREAM_POST_PROCESSING]:
+        if stream_post_processing:
             list_events = __postprocess_stream(list_events)
         for i in range(len(list_events)):
             list_events[i] = Event(list_events[i])
         log = log_instance.EventStream(list_events, attributes={'origin': 'csv'})
     if isinstance(log, EventLog):
-        case_pref = parameters[
-            Parameters.CASE_ATTRIBUTE_PREFIX] if Parameters.CASE_ATTRIBUTE_PREFIX in parameters else Parameters.CASE_ATTRIBUTE_PREFIX.value
-        enable_deepcopy = parameters[
-            Parameters.DEEP_COPY] if Parameters.DEEP_COPY in parameters else Parameters.DEEP_COPY.value
-
         return __transform_event_log_to_event_stream(log, include_case_attributes=True,
                                                      case_attribute_prefix=case_pref, enable_deepcopy=enable_deepcopy)
     return log
